% Script file to generate an animation for the IFAC NOLCOS submission by 
%   Kurzhanski, Mitchell and Varaiya.  Involves backwards reachability for
%   an ellipsoidal target set within a convex constraint set.
%
% Ian Mitchell, 7/29/04

%---------------------------------------------------------------------------
% Example parameters.
k = 1;
M = 4.0 * [ 2 0; 0 1 ];
m = [ 0.0; 0.7 ];

% Reach tube (vs reach set at a particular time).
reachTube = 0;

% How many frames in the animation?
frames = 101;

% Animation parameters
deleteLabels = 0;               % Labels tend to float around and look bad.
qualityValue = 90;		% Even 100 gets significant compression.
filename = 'nolcos';            % name to give to the avi file.
compress = 1;                   % Use lossy compression?

% What final time?
tMax = 1.0;

% Accuracy.
accuracy = 'medium';
%accuracy = 'veryHigh';

% What level set represents the boundary?
contourLevel = [ 1 1 ];

%---------------------------------------------------------------------------
% What times are the frames?
times = linspace(0, tMax, frames);
times = times(2:end);

% Without constraints.
[ dataNoClip, g, dataNoClip0 ] = ...
                                nolcos(k, M, m, times, 0, reachTube, accuracy);

% With constraints.
[ dataClip, g, dataClip0, mask ] = ...
                                nolcos(k, M, m, times, 1, reachTube, accuracy);

%---------------------------------------------------------------------------
% Make the figure.
fig = figure;

% Show the mask and the initial conditions.
[ garbage, hMask ] = contour(g.xs{1}, g.xs{2}, mask, contourLevel, 'b:');
axis equal
axis(g.axis);
hold on;

[ garbage, hClip0 ] = contour(g.xs{1}, g.xs{2}, dataClip0, contourLevel, 'k:');
[ garbage, hClip ] = contour(g.xs{1}, g.xs{2}, dataClip0, contourLevel, 'b-');
[ garbage, hNoClip ] = contour(g.xs{1}, g.xs{2}, dataNoClip0, ...
                               contourLevel, 'r--');

set([ hMask; hClip0; hNoClip ], 'LineWidth', 1.0);
set(hClip, 'LineWidth', 2.0);
title('t = 0');

%---------------------------------------------------------------------------
% Now establish the animation stuff.

% Set up the figure window nicely (you can change the resolution here).
set(fig, 'Position', [ 100 100 384 384 ], 'color', 'white');
box on

% Turn off stuff we don't want -- animations are typically too busy looking.
if(deleteLabels)
  set(gca, 'XTick', [], 'YTick', []);
  delete(get(gca, 'Title'));
  delete(get(gca, 'XLabel'));
  delete(get(gca, 'YLabel'));
end

% Create the avi file (choose a smaller qualityValue to get smaller files).
if(compress)
  mov = avifile(filename, 'quality', qualityValue);
else
  mov = avifile(filename, 'compression', 'none');
end

%---------------------------------------------------------------------------
% Capture the first frame.
drawnow;
f = getframe(gcf);
mov = addframe(mov, f);

%---------------------------------------------------------------------------
% Loop through remaining frames.
for frame = 1 : length(times)
  delete(hClip);
  delete(hNoClip);
  [ garbage, hClip ] = contour(g.xs{1}, g.xs{2}, dataClip{frame}, ...
                               contourLevel, 'b-');
  [ garbage, hNoClip ] = contour(g.xs{1}, g.xs{2}, dataNoClip{frame}, ...
                                 contourLevel, 'r--');

  set(hNoClip, 'LineWidth', 1.0);
  set(hClip, 'LineWidth', 2.0);
  title([ 't = ' num2str(times(frame)) ]);

  % Capture the next frame.
  drawnow;
  f = getframe(gcf);
  mov = addframe(mov, f);
end

%---------------------------------------------------------------------------
% We're finished with the movie.
mov = close(mov);
