% Script file to generate a figure for the IFAC NOLCOS submission by 
%   Kurzhanski, Mitchell and Varaiya.  Involves backwards reachability for
%   an ellipsoidal target set within a convex constraint set.
%
% Ian Mitchell, 4/14/04

%---------------------------------------------------------------------------
% Example parameters.
k = 1;
M = 4.0 * [ 2 0; 0 1 ];
m = [ 0.0; 0.7 ];

% Reach tube (vs reach set at a particular time).
reachTube = 0;

% How many additional time slices of the constrained set?
timeSlices = 3;

% What final time?
tMax = 1.0;

% Accuracy.
accuracy = 'medium';
%accuracy = 'veryHigh';

% What level set represents the boundary?
contourLevel = [ 1 1 ];

%---------------------------------------------------------------------------
% Without constraints.
dataNoClip = nolcos(k, M, m, tMax, 0, reachTube, accuracy);

% What times to collect?
times = linspace(0, tMax, timeSlices + 2);
times = times(2:end);

% With constraints.
[ dataClip, g, data0, mask ] = nolcos(k, M, m, times, 1, reachTube, accuracy);

%---------------------------------------------------------------------------
% Make the figure.
figure;
[ garbage, hMask ] = contour(g.xs{1}, g.xs{2}, mask, contourLevel, 'b:');
axis equal
axis(g.axis);
hold on;

[ garbage, h0 ] = contour(g.xs{1}, g.xs{2}, data0, contourLevel, 'k:');
[ garbage, hNo ] = contour(g.xs{1}, g.xs{2}, dataNoClip, contourLevel, 'r--');

if(timeSlices == 0)
  [ garbage, hClip ] = contour(g.xs{1}, g.xs{2}, dataClip, contourLevel, 'b-');
else
  for i = 1 : timeSlices
    [ garbage, hSlice(i) ] = contour(g.xs{1}, g.xs{2}, ...
                                     dataClip{i}, contourLevel, 'b-');
  end
  [ garbage, hClip ] = contour(g.xs{1}, g.xs{2}, ...
                               dataClip{end}, contourLevel, 'b-');
end

%---------------------------------------------------------------------------
% Make it pretty.
hs = [ hClip; hNo; h0; hMask ];
legend(hs, 'with constraint', 'without constraint', ...
                           'target', 'constraint');
title([ 'k = ' num2str(k) ...
        ', m = [ ' num2str(m(1)) ' ' num2str(m(2)) ...
        ' ], M = [ ' num2str(M(1,:)) '; ' num2str(M(2,:)) ' ]' ]);

set(hs, 'LineWidth', 3.0);
set(hSlice, 'LineWidth', 1.0);
